--[[
    Aaron MacDonald
    August 31, 2010
]]

require('utility')
require('levels')

collision = {}

function collision.moveRect(size, x, y, dx, dy)
    local width = size + math.abs(dx)
    local height = size + math.abs(dy)

    local rx = x - (size / 2)
    if dx < 0 then
        rx = rx + dx
    end

    local ry = y - (size / 2)
    if dy < 0 then
        ry = ry + dy
    end

    return utility.rectangle(rx, ry, width, height)
end

function collision.actorVSLevel(level, act, dx, dy)
    local resultTimeX = 1.0
    local resultTimeY = 1.0

    local size = act.type.size
    local halfSize = size / 2

    local moveRect = collision.moveRect(size, act.x, act.y, dx, dy)

    local tiles = levels.tilesInRect(level, moveRect,
      function (x, y) return levels.isWall(level, x, y) end)

    for _, t in ipairs(tiles) do
        if dx ~= 0 then
            local wallX
            local realDX

            if (dx < 0) and not levels.isWall(level, t.x + 1, t.y)
            then
                -- Going west; check east wall
                wallX = levels.pixel(t.x + 1)

                if wallX < act.x then
                    realDX = act.x - halfSize - wallX
                end
            elseif (dx > 0) and not levels.isWall(level, t.x - 1, t.y)
            then
                -- Going east; check west wall
                wallX = levels.pixel(t.x)

                if wallX > act.x then
                    realDX = wallX - act.x - halfSize
                end
            end

            if realDX  and (realDX >= 0) then
                local time = realDX / math.abs(dx)
                if time < resultTimeX then
                    resultTimeX = time
                end
            end
        end

        if dy ~= 0 then
            local wallY
            local realDY

            if (dy < 0) and not levels.isWall(level, t.x, t.y + 1)
            then
                -- Going north; check south wall
                wallY = levels.pixel(t.y + 1)

                if wallY < act.y then
                    realDY = act.y - halfSize - wallY
                end
            elseif (dy > 0) and not levels.isWall(level, t.x, t.y - 1)
            then
                -- Going south; check north wall
                wallY = levels.pixel(t.y)

                if wallY > act.y then
                    realDY = wallY - act.y - halfSize
                end
            end

            if realDY and (realDY >= 0) then
                local time = realDY / math.abs(dy)
                if time < resultTimeY then
                    resultTimeY = time
                end
            end
        end
    end

    return resultTimeX, resultTimeY
end