--[[
    Aaron MacDonald
    August 18, 2010
]]

require('graphics')
require('constants')
require('actor')
require('levels')
require('collision')

local FPS = 60
local SECONDS_PER_FRAME = 1 / FPS

local accumulatedTime

----------------------------------------------------------------------

local KEY_QUIT = 'escape'

local VALID_BUTTONS = { [constants.UP] = true,
                        [constants.DOWN] = true,
                        [constants.LEFT] = true,
                        [constants.RIGHT] = true }

----------------------------------------------------------------------

local PLAYER_TYPE = {
    spriteSheet = 'player',
    spriteWidth = 64, spriteHeight = 96,
    spriteOffsetX = 32, spriteOffsetY = 64,
    size = 64,
    speed = 16, diagonalSpeed = actor.diagonalSpeed(16)
}

local PLAYER_START_FACING = constants.DOWN

----------------------------------------------------------------------

local currentButtons = { [constants.UP] = false,
                         [constants.DOWN] = false,
                         [constants.LEFT] = false,
                         [constants.RIGHT] = false }

local player

local currentLevel

local camera

--local backgroundMusic

----------------------------------------------------------------------

function love.load()
    accumulatedTime = 0

    graphics.loadFont()

    currentLevel = levels.newLevel('test_level')

    local startX = 0
    local startY = 0

    if currentLevel.playerSpawn then
        startX = currentLevel.playerSpawn.x
        startY = currentLevel.playerSpawn.y
    end

    player = actor.newActor(PLAYER_TYPE, startX, startY,
      PLAYER_START_FACING)

    local width = love.graphics.getWidth()
    local height = love.graphics.getHeight()

    camera = {
        x = 0,
        y = 0,
        minX = width / 2,
        minY = height / 2,
        maxX = levels.pixelWidth(currentLevel) - width,
        maxY = levels.pixelHeight(currentLevel) - height,
        width = width,
        height = height
    }

    local backgroundMusic = love.audio.newSource(
      'music/Street Fighter II.ogg', 'stream')
    backgroundMusic:setLooping(true)
    love.audio.play(backgroundMusic)
end

function love.keypressed( key, unicode )
    if key == KEY_QUIT then
        love.event.push('q')
    elseif VALID_BUTTONS[key] then
        currentButtons[key] = true
    end
end

function love.keyreleased( key, unicode )
   if VALID_BUTTONS[key] then
       currentButtons[key] = false
   end
end

local function myUpdate()
    local dx, dy = actor.getVelocity(player, currentButtons)
    if (dx ~= 0) or (dy ~= 0) then
        local timeX, timeY = collision.actorVSLevel(currentLevel,
          player, dx, dy)
        actor.move(player, dx, dy, timeX, timeY)
    end

    if player.x < camera.minX then
        camera.x = 0
    elseif player.x > (camera.maxX + camera.minX) then
        camera.x = camera.maxX
    else
        camera.x = player.x - camera.minX
    end

    if (player.y - currentLevel.minY) < camera.minY then
        camera.y = currentLevel.minY
    elseif player.y > (camera.maxY + camera.minY) then
        camera.y = camera.maxY
    else
        camera.y = player.y - camera.minY
    end
end

function love.update(dt)
    accumulatedTime = accumulatedTime + dt
    while accumulatedTime >= SECONDS_PER_FRAME
    do
        myUpdate()
        accumulatedTime = accumulatedTime - SECONDS_PER_FRAME
    end
end

function love.draw()
    love.graphics.translate(-camera.x, -camera.y)

    levels.drawGround(currentLevel, camera)
    actor.draw(player)
    levels.drawOver(currentLevel, camera)

    love.graphics.translate(camera.x, camera.y)
end